@@ -262,7 +262,9 @@ KConfigPropertyMap *Applet::configuration()
 {
     if (!d->configPropertyMap) {
         d->configPropertyMap = new KConfigPropertyMap(configScheme(), this);
-        connect(d->configPropertyMap, &KConfigPropertyMap::valueChanged, this, &Applet::configNeedsSaving);
+        connect(d->configPropertyMap, &KConfigPropertyMap::valueChanged, this, [this]() {
+            d->scheduleModificationNotification();
+        });
     }
     return d->configPropertyMap;
 }
@@ -604,6 +606,17 @@ void Applet::flushPendingConstraintsEvents()
     // pass the constraint on to the actual subclass
     constraintsEvent(c);
 
+    if (c & StartupCompletedConstraint) {
+        // start up is done, we can now go do a mod timer
+        if (d->modificationsTimer) {
+            if (d->modificationsTimer->isActive()) {
+                d->modificationsTimer->stop();
+            }
+        } else {
+            d->modificationsTimer = new QBasicTimer;
+        }
+    }
+
     if (c & FormFactorConstraint) {
         Q_EMIT formFactorChanged(formFactor());
     }
@@ -831,6 +844,9 @@ void Applet::timerEvent(QTimerEvent *event)
 {
     if (d->transient) {
         d->constraintsTimer.stop();
+        if (d->modificationsTimer) {
+            d->modificationsTimer->stop();
+        }
         return;
     }
 
@@ -842,6 +858,13 @@ void Applet::timerEvent(QTimerEvent *event)
         if (!(d->pendingConstraints & StartupCompletedConstraint)) {
             flushPendingConstraintsEvents();
         }
+    } else if (d->modificationsTimer && event->timerId() == d->modificationsTimer->timerId()) {
+        d->modificationsTimer->stop();
+        // invalid group, will result in save using the default group
+        KConfigGroup cg;
+
+        save(cg);
+        Q_EMIT configNeedsSaving();
     }
 }
 
